@@ -207,6 +207,10 @@ class S7CPU(object): #+cdef
 					codeBlock = self.sfcs[codeBlockOp.value.byteOffset]
 				elif codeBlockOp.type == AwlOperator.BLKREF_SFB:
 					codeBlock = self.sfbs[codeBlockOp.value.byteOffset]
+				elif codeBlockOp.type == AwlOperator.MULTI_FB:
+					codeBlock = self.fbs[codeBlockOp.value.fbNumber]
+				elif codeBlockOp.type == AwlOperator.MULTI_SFB:
+					codeBlock = self.sfbs[codeBlockOp.value.fbNumber]
 				else:
 					raise ValueError
 			except (KeyError, ValueError) as e:
@@ -312,7 +316,6 @@ class S7CPU(object): #+cdef
 				if sfbNumber < 0 and not self.__extendedInsnsEnabled:
 					continue
 				sfb = SFB_table[sfbNumber](self)
-				sfb.interface.buildDataStructure()
 				self.sfbs[sfbNumber] = sfb
 		if not self.sfcs:
 			# Create the SFC tables
@@ -320,9 +323,12 @@ class S7CPU(object): #+cdef
 				if sfcNumber < 0 and not self.__extendedInsnsEnabled:
 					continue
 				sfc = SFC_table[sfcNumber](self)
-				sfc.interface.buildDataStructure()
 				self.sfcs[sfcNumber] = sfc
 
+		# Build the data structures of code blocks.
+		for block in self.__allCodeBlocks():
+			block.interface.buildDataStructure(self)
+
 		# Translate DBs
 		for dbNumber, rawDB in parseTree.dbs.items():
 			dbNumber, sym = resolver.resolveBlockName((AwlDataType.TYPE_DB_X,
@@ -449,8 +455,15 @@ class S7CPU(object): #+cdef
 		self.cycleStartTime = self.now
 
 		# Initialize CPU state
-		self.callStack = [ CallStackElem(self, block) ]
 		self.dbRegister = self.diRegister = self.dbs[0]
+		self.accu1.reset()
+		self.accu2.reset()
+		self.accu3.reset()
+		self.accu4.reset()
+		self.ar1.reset()
+		self.ar2.reset()
+		self.statusWord.reset()
+		self.callStack = [ CallStackElem(self, block, None, (), True) ]
 		cse = self.callStackTop = self.callStack[-1]
 		if self.__obTempPresetsEnabled:
 			# Populate the TEMP region
@@ -474,8 +487,7 @@ class S7CPU(object): #+cdef
 			prevCse = self.callStack.pop()
 			if self.callStack:
 				cse = self.callStackTop = self.callStack[-1]
-				prevCse.handleBlockExit()
-			prevCse.destroy()
+			prevCse.handleBlockExit()
 		if self.cbCycleExit:
 			self.cbCycleExit(self.cbCycleExitData)
 
@@ -668,6 +680,8 @@ class S7CPU(object): #+cdef
 		db = self.dbs[dbOper.value.byteOffset]
 		cse = CallStackElem(self, fb, db, parameters)
 		self.dbRegister, self.diRegister = self.diRegister, db
+		# Set AR2 to multi-instance base DBX 0.0
+		self.ar2.set(AwlIndirectOp.AREA_DB)
 		return cse
 
 	def __call_RAW_FB(self, blockOper, dbOper, parameters):
@@ -687,6 +701,8 @@ class S7CPU(object): #+cdef
 		db = self.dbs[dbOper.value.byteOffset]
 		cse = CallStackElem(self, sfb, db, parameters)
 		self.dbRegister, self.diRegister = self.diRegister, db
+		# Set AR2 to multi-instance base DBX 0.0
+		self.ar2.set(AwlIndirectOp.AREA_DB)
 		return cse
 
 	def __call_RAW_SFB(self, blockOper, dbOper, parameters):
@@ -702,11 +718,21 @@ class S7CPU(object): #+cdef
 			raise AwlSimError("Code block %d not found in indirect call" %\
 					  blockOper.value.byteOffset)
 
+	def __call_MULTI_FB(self, blockOper, dbOper, parameters):
+		pass#TODO
+		raise AwlSimError("Multi-instance calls not supported, yet")
+
+	def __call_MULTI_SFB(self, blockOper, dbOper, parameters):
+		pass#TODO
+		raise AwlSimError("Multi-instance calls not supported, yet")
+
 	__callHelpers = {
 		AwlOperator.BLKREF_FC	: __call_FC,
 		AwlOperator.BLKREF_FB	: __call_FB,
 		AwlOperator.BLKREF_SFC	: __call_SFC,
 		AwlOperator.BLKREF_SFB	: __call_SFB,
+		AwlOperator.MULTI_FB	: __call_MULTI_FB,
+		AwlOperator.MULTI_SFB	: __call_MULTI_SFB,
 	}
 
 	__rawCallHelpers = {
@@ -726,9 +752,8 @@ class S7CPU(object): #+cdef
 		except KeyError:
 			raise AwlSimError("Invalid CALL operand")
 		newCse = callHelper(self, blockOper, dbOper, parameters)
-		if newCse:
-			self.callStack.append(newCse)
-			self.callStackTop = newCse
+		self.callStack.append(newCse)
+		self.callStackTop = newCse
 
 	def run_BE(self):
 		s = self.statusWord
@@ -969,18 +994,17 @@ class S7CPU(object): #+cdef
 			self.run_AUF(AwlOperator(AwlOperator.BLKREF_DB, 16,
 						 AwlOffset(operator.value.dbNumber),
 						 operator.insn))
-		if not self.dbRegister:
-			raise AwlSimError("Fetch from global DB, "
-				"but no DB is opened")
 		return self.dbRegister.fetch(operator)
 
 	def fetchDI(self, operator, enforceWidth):
 		if operator.width not in enforceWidth and enforceWidth:
 			self.__fetchWidthError(operator, enforceWidth)
 
-		if not self.diRegister:
-			raise AwlSimError("Fetch from instance DI, "
-				"but no DI is opened")
+		if self.callStackTop.block.isFB:
+			# Fetch the data using the multi-instance base offset from AR2.
+			return self.diRegister.fetch(operator,
+						     AwlOffset.fromPointerValue(self.ar2.get()))
+		# Fetch without base offset.
 		return self.diRegister.fetch(operator)
 
 	def fetchPE(self, operator, enforceWidth):
@@ -1178,9 +1202,6 @@ class S7CPU(object): #+cdef
 
 		if operator.value.dbNumber is None:
 			db = self.dbRegister
-			if not db:
-				raise AwlSimError("Store to global DB, "
-					"but no DB is opened")
 		else:
 			try:
 				db = self.dbs[operator.value.dbNumber]
@@ -1193,10 +1214,13 @@ class S7CPU(object): #+cdef
 		if operator.width not in enforceWidth and enforceWidth:
 			self.__storeWidthError(operator, enforceWidth)
 
-		if not self.diRegister:
-			raise AwlSimError("Store to instance DI, "
-				"but no DI is opened")
-		self.diRegister.store(operator, value)
+		if self.callStackTop.block.isFB:
+			# Store the data using the multi-instance base offset from AR2.
+			self.diRegister.store(operator, value,
+					      AwlOffset.fromPointerValue(self.ar2.get()))
+		else:
+			# Store without base offset.
+			self.diRegister.store(operator, value)
 
 	def storePA(self, operator, value, enforceWidth):
 		if operator.width not in enforceWidth and enforceWidth:
@@ -1291,8 +1315,11 @@ class S7CPU(object): #+cdef
 		isEnglish = (mnemonics == S7CPUSpecs.MNEMONICS_EN)
 		self.updateTimestamp()
 		ret = []
-		ret.append("=== S7-CPU dump ===  (t: %.01fs)" %\
-			   (self.now - self.startupTime))
+		ret.append("=== S7-CPU dump ===  (t: %.01fs  py: %d / %s / %s)" %\
+			   (self.now - self.startupTime,
+			    3 if isPy3Compat else 2,
+			    pythonInterpreter,
+			    "Win" if osIsWindows else ("Posix" if osIsPosix else "unknown")))
 		ret.append("    STW:  " + self.statusWord.getString(mnemonics))
 		if self.is4accu:
 			accus = [ accu.toHex()
@@ -1342,8 +1369,13 @@ class S7CPU(object): #+cdef
 		ret.append("   Stmt:  IP:%s   %s" %\
 			   (str(self.getCurrentIP()),
 			    str(curInsn) if curInsn else ""))
-		ret.append("  Speed:  %d stmt/s  %.01f stmt/cycle" %\
+		if self.insnPerSecond:
+			usPerInsn = "%.03f" % ((1.0 / self.insnPerSecond) * 1000000)
+		else:
+			usPerInsn = "-/-"
+		ret.append("  Speed:  %d stmt/s (= %s us/stmt)  %.01f stmt/cycle" %\
 			   (int(round(self.insnPerSecond)),
+			    usPerInsn,
 			    self.avgInsnPerCycle))
 		ret.append(" CycleT:  avg:%.06fs  min:%.06fs  max:%.06fs" %\
 			   (self.avgCycleTime, self.minCycleTime,
