@@ -22,88 +22,13 @@
 from __future__ import division, absolute_import, print_function, unicode_literals
 from awlsim.common.compat import *
 
-#from awlsim.core.dynattrs cimport * #@cy
-
 from awlsim.common.datatypehelpers import *
 
-from awlsim.core.dynattrs import * #@nocy
 from awlsim.core.util import *
 from awlsim.core.timers import *
+from awlsim.core.offset import *
 
 
-class AwlOffset(DynAttrs): #+cdef
-	"Memory area offset"
-
-	dynAttrs = {
-		# A DB-number for fully qualified access, or None.
-		"dbNumber"	: None,
-
-		# A symbolic DB-name for fully qualified access, or None.
-		"dbName"	: None,
-
-		# A DB-variable name for fully qualified access, or None.
-		"varName"	: None,
-
-		# List of Variable indices for fully qualified array access, or None.
-		"indices"	: None,
-
-		# Additional sub-offset that is added to this offset.
-		# Defaults to 0.0
-		# This is used for arrays and structs.
-		"subOffset"	: lambda self, name: AwlOffset(0, 0),
-	}
-
-	def __init__(self, byteOffset, bitOffset=0):
-		self.byteOffset, self.bitOffset =\
-			byteOffset, bitOffset
-
-	def dup(self):
-		offset = AwlOffset(self.byteOffset,
-				   self.bitOffset)
-		offset.dbNumber = self.dbNumber
-		return offset
-
-	@classmethod
-	def fromPointerValue(cls, value):
-		return cls((value & 0x0007FFF8) >> 3,
-			   (value & 0x7))
-
-	def toPointerValue(self):
-		return ((self.byteOffset << 3) & 0x0007FFF8) |\
-		       (self.bitOffset & 0x7)
-
-	def __add__(self, other):
-		bitOffset = (self.byteOffset + other.byteOffset) * 8 +\
-			    self.bitOffset + other.bitOffset
-		return AwlOffset(bitOffset // 8, bitOffset % 8)
-
-	def __iadd__(self, other):
-		bitOffset = (self.byteOffset + other.byteOffset) * 8 +\
-			    self.bitOffset + other.bitOffset
-		self.byteOffset = bitOffset // 8
-		self.bitOffset = bitOffset % 8
-		return self
-
-	def __repr__(self):
-		prefix = ""
-		if self.dbNumber is not None:
-			prefix = "DB%d" % self.dbNumber
-		if self.dbName is not None:
-			prefix = '"%s"' % self.dbName
-		if self.varName is not None:
-			indices = ""
-			if self.indices is not None:
-				indices = "[%s]" % ",".join(str(i) for i in self.indices)
-			if prefix:
-				return prefix + "." + self.varName + indices
-			return "#%s%s" % (self.varName, indices)
-		else:
-			if prefix:
-				prefix = prefix + ".DBX "
-			return "%s%d.%d" % (prefix,
-					    self.byteOffset,
-					    self.bitOffset)
-
 class AwlDataType(object):
 	# Data type IDs
 	EnumGen.start
@@ -791,6 +716,10 @@ class GenericInteger(object): #+cdef
 #@cy	cpdef copyFrom(self, GenericInteger other):
 		self.value = other.value & self.mask
 
+	def reset(self): #@nocy
+#@cy	cpdef reset(self):
+		self.value = 0
+
 	def set(self, value): #@nocy
 #@cy	cpdef set(self, int64_t value):
 		self.value = value & self.mask
@@ -910,7 +839,8 @@ class ByteArray(bytearray):
 					raise IndexError
 				return memoryview(self)[byteOffset : byteOffset + nrBytes]
 		except IndexError as e:
-			raise AwlSimError("fetch: Operator offset out of range")
+			raise AwlSimError("fetch: Operator offset '%s' out of range" %\
+					  str(offset))
 
 	def store(self, offset, width, value): #@nocy
 #@cy	def store(self, object offset, uint32_t width, object value):
@@ -941,7 +871,8 @@ class ByteArray(bytearray):
 					raise IndexError
 				self[byteOffset : byteOffset + len(value)] = value
 		except IndexError as e:
-			raise AwlSimError("store: Operator offset out of range")
+			raise AwlSimError("store: Operator offset '%s' out of range" %\
+					  str(offset))
 
 class Accu(GenericDWord): #+cdef
 	"Accumulator register"
