@@ -20,15 +20,17 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.core.compat import *
+from awlsim.common.compat import *
+
+import sys
+import os
 
 from awlsim.gui.util import *
 from awlsim.gui.editwidget import *
 from awlsim.gui.projectwidget import *
 from awlsim.gui.cpuconfig import *
 from awlsim.gui.coreconfig import *
-
-from awlsim.coreserver.client import *
+from awlsim.gui.icons import *
 
 
 class MainWidget(QWidget):
@@ -66,6 +68,14 @@ class MainWidget(QWidget):
 	def isDirty(self):
 		return self.dirty
 
+	def setDirty(self, dirty, force=False):
+		if dirty != self.dirty or force:
+			self.dirty = dirty
+			self.dirtyChanged.emit(self.dirty)
+
+	def getFilename(self):
+		return self.filename
+
 	def __runStateChanged(self, newState):
 		self.runStateChanged.emit(newState)
 
@@ -80,10 +90,53 @@ class MainWidget(QWidget):
 
 	def __somethingChanged(self):
 		self.cpuWidget.stop()
-		self.dirty = True
-		self.dirtyChanged.emit(self.dirty)
+		self.setDirty(True)
+
+	def newFile(self, filename=None):
+		if isWinStandalone:
+			executableName = "awlsim-gui.exe"
+		else:
+			executableName = sys.executable
+		executable = findExecutable(executableName)
+		if not executable:
+			QMessageBox.critical(self,
+				"Failed to find '%s'" % executableName,
+				"Could not spawn a new instance.\n"
+				"Failed to find '%s'" % executableName)
+			return
+		if isWinStandalone:
+			argv = [ executable, ]
+		else:
+			argv = [ executable, "-m", "awlsim.gui.mainwindow", ]
+		if filename:
+			argv.append(filename)
+		PopenWrapper(argv, env = os.environ)
 
 	def loadFile(self, filename):
+		if self.dirty:
+			res = QMessageBox.question(self,
+				"Unsaved project",
+				"The current project is modified and contains unsaved changes.\n "
+				"Do you want to:\n"
+				"- Save the project, close it and open the new project\n"
+				"- Open the new project in a new instance or\n"
+				"- Discard the changes and open the new project\n"
+				"- Cancel the operation",
+				QMessageBox.Save | QMessageBox.Discard |\
+				QMessageBox.Open | QMessageBox.Cancel,
+				QMessageBox.Open)
+			if res == QMessageBox.Save:
+				if not self.save():
+					return
+			elif res == QMessageBox.Discard:
+				pass
+			elif res == QMessageBox.Open:
+				self.newFile(filename)
+				return
+			elif res == QMessageBox.Cancel:
+				return
+			else:
+				assert(0)
 		try:
 			res = self.projectWidget.loadProjectFile(filename)
 			if not res:
@@ -93,8 +146,8 @@ class MainWidget(QWidget):
 				"Failed to load project file", str(e))
 			return False
 		self.filename = filename
-		self.dirty = not bool(self.getProject().getProjectFile())
-		self.dirtyChanged.emit(self.dirty)
+		self.setDirty(dirty = not bool(self.getProject().getProjectFile()),
+			      force = True)
 		return True
 
 	def load(self):
@@ -119,9 +172,8 @@ class MainWidget(QWidget):
 			QMessageBox.critical(self,
 				"Failed to write project file", str(e))
 			return False
-		self.dirty = False
-		self.dirtyChanged.emit(self.dirty)
 		self.filename = filename
+		self.setDirty(dirty = False, force = True)
 		return True
 
 	def save(self, newFile=False):
@@ -177,25 +229,22 @@ class MainWindow(QMainWindow):
 		  initialAwlSource = None):
 		if not qApplication:
 			qApplication = QApplication(sys.argv)
-		mainwnd = cls(qApplication,
-			      initialAwlSource)
+		mainwnd = cls(initialAwlSource)
 		mainwnd.show()
 		return mainwnd
 
-	def __init__(self, qApplication, awlSource=None):
-		QMainWindow.__init__(self)
-		self.qApplication = qApplication
-
-		self.setWindowTitle("Awlsim - AWL/STL Soft-PLC v%d.%d" %\
-				    (VERSION_MAJOR, VERSION_MINOR))
+	def __init__(self, awlSource=None, parent=None):
+		QMainWindow.__init__(self, parent)
+		self.setWindowIcon(getIcon("cpu"))
 		self.setCentralWidget(MainWidget(self))
 
 		self.setMenuBar(QMenuBar(self))
 
 		menu = QMenu("&File", self)
-		menu.addAction("&Open...", self.load)
-		self.saveAct = menu.addAction("&Save", self.save)
-		menu.addAction("&Save as...", self.saveAs)
+		menu.addAction(getIcon("new"), "&New project", self.new)
+		menu.addAction(getIcon("open"), "&Open project...", self.load)
+		self.saveAct = menu.addAction(getIcon("save"), "&Save project", self.save)
+		menu.addAction(getIcon("save"), "&Save project as...", self.saveAs)
 		menu.addSeparator()
 		menu.addAction("&Exit...", self.close)
 		self.menuBar().addMenu(menu)
@@ -217,12 +266,13 @@ class MainWindow(QMainWindow):
 		self.menuBar().addMenu(menu)
 
 		menu = QMenu("Help", self)
-		menu.addAction("&About...", self.about)
+		menu.addAction(getIcon("cpu"), "&About...", self.about)
 		self.menuBar().addMenu(menu)
 
 		self.tb = QToolBar(self)
-		self.tb.addAction("Open", self.load)
-		self.tbSaveAct = self.tb.addAction("Save", self.save)
+		self.tb.addAction(getIcon("new"), "New project", self.new)
+		self.tb.addAction(getIcon("open"), "Open project", self.load)
+		self.tbSaveAct = self.tb.addAction(getIcon("save"), "Save project", self.save)
 		self.addToolBar(self.tb)
 
 		self.__dirtyChanged(False)
@@ -255,7 +305,7 @@ class MainWindow(QMainWindow):
 		self.centralWidget().insertFBcall()
 
 	def runEventLoop(self):
-		return self.qApplication.exec_()
+		return QApplication.exec_()
 
 	def getSimClient(self):
 		return self.centralWidget().getSimClient()
@@ -274,6 +324,17 @@ class MainWindow(QMainWindow):
 		self.saveAct.setEnabled(isDirty)
 		self.tbSaveAct.setEnabled(isDirty)
 
+		filename = self.centralWidget().getFilename()
+		if filename:
+			postfix = " -- " + os.path.basename(filename)
+			if isDirty:
+				postfix += "*"
+		else:
+			postfix = ""
+		self.setWindowTitle("AWL/STL Soft-PLC v%d.%d%s" %\
+				    (VERSION_MAJOR, VERSION_MINOR,
+				     postfix))
+
 	def closeEvent(self, ev):
 		cpuWidget = self.centralWidget().getCpuWidget()
 		if cpuWidget.getState() != CpuWidget.STATE_STOP:
@@ -313,6 +374,9 @@ class MainWindow(QMainWindow):
 			"any later version." %\
 			(VERSION_MAJOR, VERSION_MINOR))
 
+	def new(self):
+		self.centralWidget().newFile()
+
 	def load(self):
 		self.centralWidget().load()
 
@@ -327,3 +391,8 @@ class MainWindow(QMainWindow):
 
 	def coreConfig(self):
 		self.centralWidget().coreConfig()
+
+# If invoked as script, run a new instance.
+if __name__ == "__main__":
+	fn = sys.argv[1] if (len(sys.argv) >= 2) else None
+	sys.exit(MainWindow.start(initialAwlSource = fn).runEventLoop())
